/****************************************************************************
 Copyright (c) 2017-2018 Xiamen Yaji Software Co., Ltd.

 http://www.cocos2d-x.org

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ****************************************************************************/

#include "HelloWorldScene.h"

USING_NS_CC;

Scene* HelloWorld::createScene()
{
    return HelloWorld::create();
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
    //////////////////////////////
    // 1. super init first
    if (!Scene::init())
    {
        return false;
    }

    auto visibleSize = Director::getInstance()->getVisibleSize();
    Vec2 origin = Director::getInstance()->getVisibleOrigin();

    mSpritesLayer = Layer::create();
    mSpritesLayer->retain();

    sprite = Sprite::create("mario_3x.png");
    sprite->getTexture()->setAliasTexParameters();
    sprite->setAnchorPoint(Vec2(0, 1));
    sprite->setScale(visibleSize.height / mDesignSize.height);
    sprite->setPositionY(visibleSize.height);
    sprite->setPositionX(0);
    mSpritesLayer->addChild(sprite);


    // RenderTexture
    mRTex = RenderTexture::create(visibleSize.width, visibleSize.height);
    mRTex->setPosition(visibleSize.width / 2, visibleSize.height / 2);
    this->addChild(mRTex, 100);


    mNodeGrid = NodeGrid::create(Rect(origin,visibleSize));
    mNodeGrid->retain();
    mNodeGrid->setTarget(mSpritesLayer);
    
    mFlipY3dAction = FlipY3D::create(1);
    mFlipY3dAction->retain();

    return true;
}

void HelloWorld::draw(cocos2d::Renderer* renderer, const cocos2d::Mat4& transform, uint32_t flags)
{
    if (mTick == 50)
    {
        mNodeGrid->runAction(mFlipY3dAction);
        mNodeGrid->resume();
    }

    mTick++;

    Scene::draw(renderer, transform, flags);

    // The code below works fine when RenderTexture is not used
    mRTex->beginWithClear(0, 0, 0, 255); // comment out and it works
    
    mNodeGrid->visit(renderer, transform, flags); // maybe transform issue?

    mRTex->end(); // comment out and it works
}
